//
//  CPUUsageMonitor.h
//  CPU Usage
//
//  Created by Peter Hosey on 2006-06-21.
//  Copyright 2006 Peter Hosey. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#include <mach/mach.h>
#include <mach/processor_info.h>
#include <mach/mach_host.h>

enum CPUUsageOrientation { CPUUsageOrientationVertical, CPUUsageOrientationHorizontal };

@interface CPUUsageMonitor : NSObject {
	NSWindow *window;

	//Preferences views.
	IBOutlet NSPanel *preferencesPanel;
	NSColor *backgroundColor;
	float cellWidth, cellHeight;
	IBOutlet NSTextField *widthField, *heightField;
	IBOutlet NSButton *horizontalButton, *verticalButton;

	IBOutlet NSMenu *contextualMenu;

	NSMutableArray *usageViews;
	enum CPUUsageOrientation orientation;

	NSTimer *updateTimer;
	processor_info_array_t lastProcessorInfo;
	mach_msg_type_number_t numLastProcessorInfo;
	float *CPUUsage;
	NSLock *CPUUsageLock;
	BOOL threadsShouldExit;

	//Saved preferences, for restoration after Cancel.
	NSColor *savedBackgroundColor;
	NSSize   savedCellSize;
	enum CPUUsageOrientation savedOrientation;
}

#pragma mark Accessors

- (NSColor *)backgroundColor;
- (void)setBackgroundColor:(NSColor *)newBackgroundColor;

- (float)cellWidth;
- (void)setCellWidth:(float)newCellWidth;

- (float)cellHeight;
- (void)setCellHeight:(float)newCellHeight;

#pragma mark Actions

- (IBAction)setOrientationToVertical:sender;
- (IBAction)setOrientationToHorizontal:sender;

@end
