//
//  CPUUsageView.m
//  CPU Usage
//
//  Created by Peter Hosey on 2006-06-21.
//  Copyright 2006 Peter Hosey. All rights reserved.
//

#import "CPUUsageView.h"

#import "NSString+Percentage.h"

#include <c.h>

@implementation CPUUsageView

- initWithFrame:(NSRect)newFrame {
	if((self = [super initWithFrame:newFrame])) {
		float widthScale  = newFrame.size.width  / 32.0f;
		float heightScale = newFrame.size.height / 32.0f;

		float pointSize = [NSFont smallSystemFontSize] * MIN(widthScale, heightScale);

		drawingFont = [[NSFont boldSystemFontOfSize:pointSize] retain];
		[self setBackgroundColor:[NSColor redColor]];
		NSMutableParagraphStyle *pStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
		[pStyle setAlignment:NSCenterTextAlignment];
		drawingAttributes = [[NSMutableDictionary alloc] initWithObjectsAndKeys:
			drawingFont, NSFontAttributeName,
			backgroundColor, NSForegroundColorAttributeName,
			pStyle, NSParagraphStyleAttributeName,
			nil];
		[pStyle release];

		[self setCPUUsage:0.5f];
		[self setDrawsFrame:YES];

		//Compute the width and height fractions.
		NSSize astrSize = [@"100%" sizeWithAttributes:drawingAttributes];
		textFractionOfWidth  = astrSize.width  / 32.0f;
		textFractionOfHeight = astrSize.height / 32.0f;
		scaleFromLineHeightToFontSize = astrSize.height / pointSize;
	}
	return self;
}
- (void)dealloc {
	[backgroundColor release];
	[backgroundColorPremultiplied release];

	[drawingFont release];
	[drawingAttributes release];

	[super dealloc];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
	if([keyPath isEqualToString:@"backgroundColor"])
		[self setBackgroundColor:[change objectForKey:NSKeyValueChangeNewKey]];
}

#pragma mark Graphics

- (void)drawRect:(NSRect)rect {
	NSRectClip(rect);
	NSRect bounds = [self bounds];

	[backgroundColorPremultiplied setFill];
	//For some reason, NSRectFill doesn't work here.
//	NSRectFill(rect);
	[NSBezierPath fillRect:rect]; //Not bounds, solely because we can get away with it.

	if(drawFrame) {
		[backgroundColor set]; //Not setStroke, for unknown reasons.
		NSFrameRectWithWidth(bounds, 2.0f);
	}

	NSString *str = [NSString stringWithPercentageFromFraction:CPUUsage CPUNumber:CPUNumber];
	NSAttributedString *astr = [[NSAttributedString alloc] initWithString:str attributes:drawingAttributes];

	NSSize astrSize = [astr size];
	NSImage *percentageImage = [[NSImage alloc] initWithSize:astrSize];
	[percentageImage lockFocus];
	[astr drawInRect:(NSRect){ NSZeroPoint, astrSize }];
	[percentageImage unlockFocus];

	NSPoint drawingPoint = {
		bounds.size.width  * 0.5f - astrSize.width  * 0.5f,
		bounds.size.height * 0.5f - astrSize.height * 0.5f
	};
	[percentageImage drawAtPoint:drawingPoint fromRect:NSZeroRect operation:NSCompositeDestinationOut fraction:1.0f];

	[percentageImage release];

	[astr release];
}

- (void)setFrame:(NSRect)newFrame {
	[super setFrame:newFrame];

	float widthScale  = newFrame.size.width  / 32.0f;
	float heightScale = newFrame.size.height / 32.0f;

	float newPointSize = [NSFont smallSystemFontSize] * MIN(widthScale, heightScale);
	[drawingAttributes setObject:[NSFont boldSystemFontOfSize:newPointSize] forKey:NSFontAttributeName];
}

#pragma mark Accessors

- (NSColor *)backgroundColor {
	return backgroundColor;
}
- (void)setBackgroundColor:(NSColor *)newBackgroundColor {
	if(backgroundColor != newBackgroundColor) {
		[backgroundColor release];
		backgroundColor = [newBackgroundColor retain];

		[drawingAttributes setObject:backgroundColor forKey:NSForegroundColorAttributeName];

		[backgroundColorPremultiplied release];
		backgroundColorPremultiplied = [[backgroundColor colorWithAlphaComponent:CPUUsage] retain];
	}
}

- (BOOL)drawsFrame {
	return drawFrame;
}
- (void)setDrawsFrame:(BOOL)flag {
	drawFrame = flag;
}

- (unsigned)CPUNumber {
	return CPUNumber;
}
- (void)setCPUNumber:(unsigned)newCPUNumber {
	CPUNumber = newCPUNumber;

	[self setNeedsDisplay:YES];
}

- (float)CPUUsage {
	return CPUUsage;
}
- (void)setCPUUsage:(float)newCPUUsage {
	CPUUsage = newCPUUsage;

	[backgroundColorPremultiplied release];
	backgroundColorPremultiplied = [[backgroundColor colorWithAlphaComponent:CPUUsage] retain];
}

@end
