//
//  NSString+Percentage.m
//  CPU Usage
//
//  Created by Peter Hosey on 2006-06-24.
//  Copyright 2006 Peter Hosey. All rights reserved.
//

#import "NSString+Percentage.h"

@implementation NSString (Percentage)

+ (NSString *)stringWithPercentageFromFraction:(float)fraction {
	return [self stringWithPercentageFromFraction:fraction CPUNumber:0U];
}
+ (NSString *)stringWithPercentageFromFraction:(float)fraction CPUNumber:(unsigned)CPUNumber {
	//Note: stringWithFormat: is slow.
//	return [NSString stringWithFormat:@"%.0f%%", CPUUsage * 100.0];
	//Thanks to Graham Booker for his help with optimizing this stringification code. <http://paste.lisp.org/display/21674#1>

	//Worst-case scenario: CPUNumber = 4294967295
	//Actually, there could be a percentage > 100%, but I'm not *that* paranoid. ;)
	char buf[16] = {
		0, 0, 0, 0, //100%
		0, 0, 0, 0, //NL 429
		0, 0, 0, 0, //4967
		0, 0, 0, 0, //295 NUL
	};

	//First, convert the percentage.
	register char *ptr = buf + 3;
	register unsigned percentage = fraction * 100.0f;
	*ptr = '%';
	*--ptr = '0' + percentage % 10U;
	percentage /= 10U;
	if(percentage > 0U) {
		*--ptr = '0' + percentage % 10U;
		percentage /= 10U;
	}
	if(percentage > 0U)
		*--ptr = '0' + percentage % 10U;
	const char *startPtr = ptr;

	//Add the CPU number, if there is one. 0U = no CPU number.
	if(CPUNumber) {
		buf[4] = '\n';
		ptr = buf + 4;
		register unsigned tmp = CPUNumber;
		while(tmp) {
			++ptr;
			tmp /= 10U;
		}
		tmp = CPUNumber;
		while(tmp) {
			*(ptr--) = '0' + (tmp % 10U);
			tmp /= 10U;
		}
	}

	return [NSString stringWithUTF8String:startPtr];
}

@end
